/****************************************************************************
*
*						  Ultra Long Period Timer
*
*                   Copyright (C) 1993-4 SciTech Software
*							All rights reserved.
*
* Filename:		$RCSfile: ulztimer.c $
* Version:		$Revision: 1.1 $
*
* Language:		ANSI C
* Environment:	IBM PC (MSDOS and Windows)
*
* Description:	Module to interface to the BIOS Timer Tick for timing
*				code that takes up to 24 hours (ray tracing etc). There
*				is a small overhead in calculating the time, this
*				will be negligible for such long periods of time.
*
* $Id: ulztimer.c 1.1 1995/02/05 13:14:55 kjb release $
*
****************************************************************************/

#if	defined(WINDPMI16) || defined(WINDPMI32)
#define	WINDPMI
#include <windows.h>
#include <toolhelp.h>
#endif
#include "pmode.h"
#include "ztimer.h"

#ifdef	WINDPMI

/* Windows compatability routines. Rather than reading the timer directly
 * under Windows, we use the Windows API routines (which only provide an
 * accuracy of 1ms however). The timer interrupts are virtualised under
 * windows, which causes us to get incorrect results.
 */

PRIVATE	TIMERINFO	tmStart,tmEnd,tmLap;

void _cdecl LZTimerOn(void)
{
	tmStart.dwSize = tmEnd.dwSize = tmLap.dwSize = sizeof(TIMERINFO);
	TimerCount(&tmStart);
}

ulong _cdecl LZTimerLap(void)
{
	TimerCount(&tmLap);
	return (tmLap.dwmsSinceStart - tmStart.dwmsSinceStart) * 1000L;
}

void _cdecl LZTimerOff(void)
{
	TimerCount(&tmEnd);
}

ulong _cdecl LZTimerCount(void)
{
	return (tmEnd.dwmsSinceStart - tmStart.dwmsSinceStart) * 1000L;
}

#endif

PRIVATE	ulong	start,finish;
ushort 			_ZTimerBIOS;

void ZTimerInit(void)
{ _ZTimerBIOS = PM_getBIOSSelector(); }

void ULZTimerOn(void)
{ start = ULZReadTime(); }

void ULZTimerOff(void)
{ finish = ULZReadTime(); }

ulong ULZTimerLap(void)
{ return ULZElapsedTime(start,ULZReadTime()); }

ulong ULZTimerCount(void)
{ return ULZElapsedTime(start,finish); }

ulong ULZReadTime(void)
/****************************************************************************
*
* Function:		ULZReadTime
* Returns:		Current timer tick count.
*
* Description:	We turn of interrupts while we
*				get the value from the BIOS data area since it is stored
*				as two bytes and an interrupt COULD stuff up our reading.
*
****************************************************************************/
{
	ulong	ticks;

	_disable();				/* Turn of interrupts 				*/
	ticks = PM_getLong(_ZTimerBIOS,0x6C);
	_enable();				/* Turn on interrupts again 		*/

	return ticks;
}

ulong ULZElapsedTime(ulong start,ulong finish)
/****************************************************************************
*
* Function:		ULZElapsedTime
* Parameters:	start	- Starting timer tick value
*				finish	- Ending timer tick value
*
* Returns:		Elapsed timer between starting time and ending time in
*				1/18 ths of a second.
*
****************************************************************************/
{
	/* Check to see whether a midnight boundary has passed, and if so
	 * adjust the finish time to account for this. We cannot detect if
	 * more that one midnight boundary has passed, so if this happens
	 * we will be generating erronous results.
	 */

	if (finish < start)
		finish += 1573040L;			/* Number of ticks in 24 hours		*/

	return finish - start;
}
